#ifndef __SIMPLE_TARGETS_LLVM_IR_TARGET_H__
#define __SIMPLE_TARGETS_LLVM_IR_TARGET_H__

#include <cdk/targets/basic_target.h>
#include <cdk/ast/basic_node.h>
#include "targets/llvm_ir_writer.h"

#include <llvm/IR/Module.h>
#include <llvm/IR/LLVMContext.h>
#include <llvm/Support/raw_os_ostream.h>

namespace simple {

  class llvm_ir_target: public cdk::basic_target {
    static llvm_ir_target _self;

  private:
    llvm_ir_target() :
        cdk::basic_target("ll") {
    }

  public:
    bool evaluate(std::shared_ptr<cdk::compiler> compiler) {
      // this symbol table will be used to check identifiers
      // during code generation
      cdk::symbol_table<simple::symbol> symtab;

      llvm::LLVMContext context;
      llvm::Module module("this module", context);

      llvm_ir_writer writer(compiler, symtab, &module);
      compiler->ast()->accept(&writer, 0);

      llvm::raw_os_ostream rss(*compiler->ostream());
      module.print(rss, nullptr);

      return true;
    }

  };

} // simple

#endif
